package sample.bloommaker_process_and_task_form;

import java.util.HashMap;
import java.util.Map;

import jp.co.intra_mart.activiti.engine.ActivitiException;
import jp.co.intra_mart.activiti.engine.ProcessEngine;
import jp.co.intra_mart.activiti.engine.ProcessEngineFactory;
import jp.co.intra_mart.activiti.engine.history.HistoricProcessInstance;
import jp.co.intra_mart.activiti.engine.history.HistoricProcessInstanceQuery;
import jp.co.intra_mart.common.aid.jdk.java.lang.StringUtil;
import jp.co.intra_mart.foundation.activiti.extension.authority.BPMAuthorityHelper;
import jp.co.intra_mart.foundation.bloommaker.exception.BloomMakerException;
import jp.co.intra_mart.foundation.bloommaker.exception.BloomMakerPreprocessException;
import jp.co.intra_mart.foundation.bloommaker.route.preprocess.BMContentPreprocessor;
import jp.co.intra_mart.foundation.bloommaker.route.preprocess.BMContentPreprocessorContext;
import jp.co.intra_mart.foundation.security.exception.AccessSecurityException;
import jp.co.intra_mart.foundation.security.message.MessageManager;
import jp.co.intra_mart.foundation.tenant.safe_url.SafeUrlManager;
import jp.co.intra_mart.foundation.tenant.safe_url.SafeUrlManagerFactory;

public class BPMProcessStartPreprocessor implements BMContentPreprocessor {

    @Override
    public Map<String, Object> execute(final BMContentPreprocessorContext context) throws BloomMakerException {
        final MessageManager messageManager = MessageManager.getInstance();
        try {
            // コールバックパスの妥当性を検証します。
            final String callbackPath = (String) context.getParsedRequestParameters().get("callbackPath");
            if (!checkCallbackPath(callbackPath)) {
                throw new BloomMakerPreprocessException(messageManager.getMessage("MSG.E.BPM.SAMPLE.PARAM.INVALID.CALLBACK.PATH", callbackPath));
            }

            // 必須パラメータの確認とユーザ権限の妥当性を検証します。
            final String processDefinitionId = (String) context.getParsedRequestParameters().get("processDefinitionId");
            final String historicProcessInstanceId = (String) context.getParsedRequestParameters().get("historicProcessInstanceId");
            if (!StringUtil.isBlank(processDefinitionId) && !StringUtil.isBlank(historicProcessInstanceId)) {
                throw new BloomMakerPreprocessException(messageManager.getMessage("MSG.E.BPM.SAMPLE.PARAM.INVALID.REQUIRE.ONE", "processDefinitionId", "historicProcessInstanceId"));
            }

            boolean isReference = false;
            if (!StringUtil.isBlank(processDefinitionId)) {
                // プロセス開始時の権限チェックを行います。
                if (!BPMAuthorityHelper.canStartProcess(processDefinitionId)) {
                    throw new BloomMakerPreprocessException(messageManager.getMessage("MSG.E.BPM.SAMPLE.NOT.AUTHORITY.OPERATE"));
                }
            } else if (!StringUtil.isBlank(historicProcessInstanceId)) {
                // 履歴参照時の権限チェックを行います。
                if (!BPMAuthorityHelper.canReferProcessInstance(historicProcessInstanceId)) {
                    throw new BloomMakerPreprocessException(messageManager.getMessage("MSG.E.BPM.SAMPLE.NOT.AUTHORITY.OPERATE"));
                }
                isReference = true;
            } else {
                throw new BloomMakerPreprocessException(messageManager.getMessage("MSG.E.BPM.SAMPLE.PARAM.INVALID.REQUIRE.ONE", "processDefinitionId", "historicProcessInstanceId"));
            }

            HashMap<String, Object> rtn = new HashMap<>();
            rtn.put("isReference", isReference);
            if (isReference) {
                try {
                    ProcessEngine processEngine = ProcessEngineFactory.getInstance().getProcessEngine();
                    HistoricProcessInstanceQuery historicProcessInstanceQuery = processEngine.getHistoryService().createHistoricProcessInstanceQuery();
                    HistoricProcessInstance historicProcessInstance = historicProcessInstanceQuery.processInstanceId(historicProcessInstanceId).includeProcessVariables().singleResult();
                    Map<String, Object> procVars = historicProcessInstance.getProcessVariables();
                    rtn.put("businessKey", procVars.get("start-event_1_businessKey"));
                    rtn.put("assignee", procVars.get("start-event_1_assignee"));
                    rtn.put("assigneeName", procVars.get("start-event_1_assigneeName"));
                    rtn.put("description", procVars.get("start-event_1_description"));
                } catch (ActivitiException ae) {
                    throw new BloomMakerPreprocessException(messageManager.getMessage("MSG.E.BPM.SAMPLE.ERROR.EXCEPTION"), ae);
                }
            }
            return rtn;
        } catch (final AccessSecurityException e) {
            // messageManager#getMessage
            // 検査例外「アクセスセキュリティ例外クラス（AccessSecurityException）」を補足した場合、
            // BloomMakerExceptionでラップしスローします。
            throw new BloomMakerException(e);
        }
    }

    private boolean checkCallbackPath(final String callbackPath) {
        if (StringUtil.isBlank(callbackPath)) {
            return true;
        }
        final SafeUrlManager manager = SafeUrlManagerFactory.getInstance().getSafeUrlManager();
        return manager.isSafe(callbackPath);
    }

}
