package sample.bloommaker_process_and_task_form;

import java.util.HashMap;
import java.util.Map;

import jp.co.intra_mart.activiti.engine.ActivitiException;
import jp.co.intra_mart.activiti.engine.ProcessEngine;
import jp.co.intra_mart.activiti.engine.ProcessEngineFactory;
import jp.co.intra_mart.activiti.engine.history.HistoricTaskInstance;
import jp.co.intra_mart.activiti.engine.history.HistoricTaskInstanceQuery;
import jp.co.intra_mart.activiti.engine.task.Task;
import jp.co.intra_mart.activiti.engine.task.TaskQuery;
import jp.co.intra_mart.common.aid.jdk.java.lang.StringUtil;
import jp.co.intra_mart.foundation.activiti.extension.authority.BPMAuthorityHelper;
import jp.co.intra_mart.foundation.bloommaker.exception.BloomMakerException;
import jp.co.intra_mart.foundation.bloommaker.exception.BloomMakerPreprocessException;
import jp.co.intra_mart.foundation.bloommaker.route.preprocess.BMContentPreprocessor;
import jp.co.intra_mart.foundation.bloommaker.route.preprocess.BMContentPreprocessorContext;
import jp.co.intra_mart.foundation.security.exception.AccessSecurityException;
import jp.co.intra_mart.foundation.security.message.MessageManager;
import jp.co.intra_mart.foundation.tenant.safe_url.SafeUrlManager;
import jp.co.intra_mart.foundation.tenant.safe_url.SafeUrlManagerFactory;

public class BPMTaskCompletePreprocessor implements BMContentPreprocessor {

    @Override
    public Map<String, Object> execute(final BMContentPreprocessorContext context) throws BloomMakerException {
        final MessageManager messageManager = MessageManager.getInstance();
        try {
            // コールバックパスの妥当性を検証します。
            final String callbackPath = (String) context.getParsedRequestParameters().get("callbackPath");
            if (!checkCallbackPath(callbackPath)) {
                throw new BloomMakerPreprocessException(messageManager.getMessage("MSG.E.BPM.SAMPLE.PARAM.INVALID.CALLBACK.PATH", callbackPath));
            }

            // 必須パラメータの確認とユーザ権限の妥当性を検証します。
            final String taskId = (String) context.getParsedRequestParameters().get("taskId");
            final String historicTaskId = (String) context.getParsedRequestParameters().get("historicTaskId");
            if (!StringUtil.isBlank(taskId) && !StringUtil.isBlank(historicTaskId)) {
                throw new BloomMakerPreprocessException(messageManager.getMessage("MSG.E.BPM.SAMPLE.PARAM.INVALID.REQUIRE.ONE", "taskId", "historicTaskId"));
            }

            boolean isReference = false;
            if (!StringUtil.isBlank(taskId)) {
                // タスク開始時の権限チェックを行います。
                if (!BPMAuthorityHelper.canCompleteTask(taskId)) {
                    throw new BloomMakerPreprocessException(messageManager.getMessage("MSG.E.BPM.SAMPLE.NOT.AUTHORITY.OPERATE"));
                }
            } else if (!StringUtil.isBlank(historicTaskId)) {
                // タスク参照時の権限チェックを行います。
                if (!BPMAuthorityHelper.canReferTask(historicTaskId)) {
                    throw new BloomMakerPreprocessException(messageManager.getMessage("MSG.E.BPM.SAMPLE.NOT.AUTHORITY.OPERATE"));
                }
                isReference = true;
            } else {
                throw new BloomMakerPreprocessException(messageManager.getMessage("MSG.E.BPM.SAMPLE.PARAM.INVALID.REQUIRE.ONE", "taskId", "historicTaskId"));
            }

            // 初期表示情報の取得
            Map<String, Object> rtn = new HashMap<>();
            rtn.put("isReference", isReference);
            ProcessEngine processEngine = ProcessEngineFactory.getInstance().getProcessEngine();
            try {
                if (isReference) {
                    // 履歴参照時
                    HistoricTaskInstanceQuery historicTaskQuery = processEngine.getHistoryService().createHistoricTaskInstanceQuery();
                    HistoricTaskInstance historicTask = historicTaskQuery.taskId(historicTaskId).includeProcessVariables().singleResult();
                    Map<String, Object> procVars = historicTask.getProcessVariables();
                    rtn.put("assignee", (String) procVars.get(historicTaskId + "_assignee2"));
                    rtn.put("assigneeName", (String) procVars.get(historicTaskId + "_assigneeName2"));
                    rtn.put("description", (String) procVars.get(historicTaskId + "_description"));
                } else {
                    // taskの処理時
                    TaskQuery taskQuery = processEngine.getTaskService().createTaskQuery();
                    Task task = taskQuery.taskId(taskId).includeProcessVariables().singleResult();
                    Map<String, Object> procVars = task.getProcessVariables();
                    rtn.put("assignee", (String) procVars.get("assignee"));
                    rtn.put("assigneeName", (String) procVars.get("assigneeName"));
                    rtn.put("description", (String) procVars.get("description"));
                }
            } catch (ActivitiException ae) {
                throw new BloomMakerPreprocessException(messageManager.getMessage("MSG.E.BPM.SAMPLE.ERROR.EXCEPTION"), ae);
            }
            return rtn;
        } catch (final AccessSecurityException e) {
            // messageManager#getMessage
            // 検査例外「アクセスセキュリティ例外クラス（AccessSecurityException）」を補足した場合、
            // BloomMakerExceptionでラップしスローします。
            throw new BloomMakerException(e);
        }

    }

    private boolean checkCallbackPath(final String callbackPath) {
        if (StringUtil.isBlank(callbackPath)) {
            return true;
        }

        final SafeUrlManager manager = SafeUrlManagerFactory.getInstance().getSafeUrlManager();
        return manager.isSafe(callbackPath);
    }

}
